@@ -14,7 +14,7 @@ my $prefix = __PACKAGE__;
 # this module import config
 #
 use Carp ();
-use Net::SNMP::Mixin::Util qw/idx2val normalize_mac/;
+use Net::SNMP::Mixin::Util qw/idx2val normalize_mac get_init_slot/;
 
 #
 # this module export config
@@ -32,8 +32,8 @@ BEGIN {
 }
 
 use Sub::Exporter -setup => {
-  exports   => [@mixin_methods],
-  groups    => { default => [@mixin_methods], },
+  exports => [@mixin_methods],
+  groups  => { default => [@mixin_methods], },
 };
 
 #
@@ -44,7 +44,7 @@ use constant {
   DOT1D_BASE_NUM_PORTS      => '1.3.6.1.2.1.17.1.2.0',
   DOT1D_BASE_TYPE           => '1.3.6.1.2.1.17.1.3.0',
 
-  DOT1D_BASE_PORT_IF_INDEX                => '1.3.6.1.2.1.17.1.4.1.2',
+  DOT1D_BASE_PORT_IF_INDEX => '1.3.6.1.2.1.17.1.4.1.2',
 };
 
 =head1 NAME
@@ -53,18 +53,18 @@ Net::SNMP::Mixin::Dot1dBase - mixin class for the switch dot1d base values
 
 =head1 VERSION
 
-Version 0.04
+Version 0.06
 
 =cut
 
-our $VERSION = '0.04';
+our $VERSION = '0.06';
 
 =head1 SYNOPSIS
 
 A Net::SNMP mixin class for Dot1d base info.
 
   use Net::SNMP;
-  use Net::SNMP::Mixin qw/mixer init_mixins/;
+  use Net::SNMP::Mixin;
 
   # class based mixin
   Net::SNMP->mixer('Net::SNMP::Mixin::Dot1dBase');
@@ -76,7 +76,8 @@ A Net::SNMP mixin class for Dot1d base info.
   $session->mixer('Net::SNMP::Mixin::Dot1dBase');
   $session->init_mixins;
   snmp_dispatcher() if $session->nonblocking;
-  die $session->error if $session->error;
+  $session->init_ok;
+  die $session->errors if $session->errors;
 
   my $base_group = $session->get_dot1d_base_group;
 
@@ -115,8 +116,10 @@ Returns the dot1dBase group as a hash reference:
 
 sub get_dot1d_base_group {
   my $session = shift;
-  Carp::croak "'$prefix' not initialized,"
-    unless $session->{$prefix}{__initialized};
+  my $agent   = $session->hostname;
+
+  Carp::croak "$agent: '$prefix' not initialized,"
+    unless $session->init_ok($prefix);
 
   my $result = { %{ $session->{$prefix}{dot1dBase} } };
 
@@ -140,8 +143,10 @@ Returns a reference to a hash with the following entries:
 
 sub map_bridge_ports2if_indexes {
   my ( $session, ) = @_;
-  Carp::croak "'$prefix' not initialized,"
-    unless $session->{$prefix}{__initialized};
+  my $agent = $session->hostname;
+
+  Carp::croak "$agent: '$prefix' not initialized,"
+    unless $session->init_ok($prefix);
 
   # datastructure:
   # $session->{$prefix}{dot1dBasePortIfIndex}{$dot1d_base_port} = ifIndex
@@ -171,8 +176,10 @@ Returns a reference to a hash with the following entries:
 
 sub map_if_indexes2bridge_ports {
   my ( $session, ) = @_;
-  Carp::croak "'$prefix' not initialized,"
-    unless $session->{$prefix}{__initialized};
+  my $agent = $session->hostname;
+
+  Carp::croak "$agent: '$prefix' not initialized,"
+    unless $session->init_ok($prefix);
 
   # datastructure:
   # $session->{$prefix}{dot1dBasePortIfIndex}{$dot1d_base_port} = ifIndex
@@ -199,11 +206,23 @@ Fetch the dot1d base related snmp values from the host. Don't call this method d
 
 =cut
 
-sub _init {
-  my ($session, $reload) = @_;
+#
+# due to the asynchron nature, we don't know what init job is really the last, we decrement
+# the value after each callback
+#
+use constant THIS_INIT_JOBS => 2;
 
-  die "$prefix already initalized and reload not forced.\n"
-  	if $session->{$prefix}{__initialized} && not $reload;
+sub _init {
+  my ( $session, $reload ) = @_;
+  my $agent = $session->hostname;
+
+  die "$agent: $prefix already initialized and reload not forced.\n"
+    if exists get_init_slot($session)->{$prefix}
+      && get_init_slot($session)->{$prefix} == 0
+      && not $reload;
+
+  # set number of async init jobs for proper initialization
+  get_init_slot($session)->{$prefix} = THIS_INIT_JOBS;
 
   # initialize the object for dot1dbase infos
   _fetch_dot1d_base($session);
@@ -267,15 +286,17 @@ sub _dot1d_base_cb {
   return unless defined $vbl;
 
   $session->{$prefix}{dot1dBase}{dot1dBaseBridgeAddress} =
-    $vbl->{DOT1D_BASE_BRIDGE_ADDRESS()};
+    $vbl->{ DOT1D_BASE_BRIDGE_ADDRESS() };
 
   $session->{$prefix}{dot1dBase}{dot1dBaseNumPorts} =
-    $vbl->{DOT1D_BASE_NUM_PORTS()};
+    $vbl->{ DOT1D_BASE_NUM_PORTS() };
 
-  $session->{$prefix}{dot1dBase}{dot1dBaseType} =
-    $vbl->{DOT1D_BASE_TYPE() };
+  $session->{$prefix}{dot1dBase}{dot1dBaseType} = $vbl->{ DOT1D_BASE_TYPE() };
 
-  $session->{$prefix}{__initialized}++;
+  # this init job is finished
+  get_init_slot($session)->{$prefix}--;
+
+  return 1;
 }
 
 =head2 B<< _fetch_dot1d_base_ports($session) >>
@@ -290,7 +311,7 @@ sub _fetch_dot1d_base_ports {
 
   # fetch the dot1dBasePorts, in blocking or nonblocking mode
   $result = $session->get_entries(
-    -columns => [DOT1D_BASE_PORT_IF_INDEX,],
+    -columns => [ DOT1D_BASE_PORT_IF_INDEX, ],
 
     # define callback if in nonblocking mode
     $session->nonblocking ? ( -callback => \&_dot1d_base_ports_cb ) : (),
@@ -321,7 +342,10 @@ sub _dot1d_base_ports_cb {
   $session->{$prefix}{dot1dBasePortIfIndex} =
     idx2val( $vbl, DOT1D_BASE_PORT_IF_INDEX );
 
-  $session->{$prefix}{__initialized}++;
+  # this init job is finished
+  get_init_slot($session)->{$prefix}--;
+
+  return 1;
 }
 
 =head1 REQUIREMENTS
@@ -342,7 +366,7 @@ Karl Gaissmaier <karl.gaissmaier at uni-ulm.de>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2008 Karl Gaissmaier, all rights reserved.
+Copyright 2008-2016 Karl Gaissmaier, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
